var should=require("should");
var assert=require("assert");
var path=require("path");

var TDPJSObjectMerge=require(path.join(__dirname, "../TDPJSObjectMerge.js"));

var tjm=new TDPJSObjectMerge();

var objA=
{
	propBool:true,
	propInt:1,
	propStr:"This is a string v1",
	propObj:
	{
		ca:"cb", c1:123
	},
	propArr:[1,2,3,4,5,6,7,8,9],
	propFunc:function pfa(){return true;}
};

var objB=
{
	propBool:false,
	propInt:-1,
	propStr:"This is a string v2",
	propObj:
	{
		ca:"bc", c1:321
	},
	propArr:[9,8,7,6,5,4,3,2,1],
	propFunc:function pfb(){return false;}
};


describe("TDPJSObjectMerge simple (2 object) tests", function()
{
	it("Should return objA when merging objA with an empty object with ObjA having precedence (last arg)", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge({},objA);

		assert.deepEqual(res, objA);
	});	

	it("Should return objA when merging objA with an empty object with ObjA not having precedence (first arg)", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge(objA, {});

		assert.deepEqual(res, objA);
	});	

	it("Should return objA when merging objA with an objB when objA has precedence (last arg)", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge(objB,objA);

		assert.deepEqual(res, objA);
	});

	it("Should return objB when merging objA with an objB when objB has precedence (last arg)", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge(objA,objB);

		assert.deepEqual(res, objB);
	});


	it("Should return objA with bool mod when merging objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge(objA,{propBool:false});

		res.propBool.should.eql(false);

		res.propInt.should.eql(objA.propInt);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

	it("Should return objA with int mod when merging objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge(objA,{propInt:2});

		res.propInt.should.eql(2);

		res.propBool.should.eql(objA.propBool);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

	it("Should return objA with str mod when merging objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
		
		var res=tjm.merge(objA,{propStr:"Another string"});

		res.propStr.should.eql("Another string");

		res.propInt.should.eql(objA.propInt);
		res.propBool.should.eql(objA.propBool);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});


	it("Should return objA with obj mod when merging objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};
				
		var res=tjm.merge(objA,{propObj:{z:"x"}});

		assert.deepEqual(res.propObj, { ca: 'cb', c1: 123, z: 'x' });

		res.propInt.should.eql(objA.propInt);
		res.propBool.should.eql(objA.propBool);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});


	it("Should return objA with array mod when merging objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};

		var res=tjm.merge(objA,{propArr:["a","b"]});

		assert.deepEqual(res.propArr, ['a', 'b', 3, 4, 5, 6, 7, 8, 9 ]);

		res.propInt.should.eql(objA.propInt);
		res.propBool.should.eql(objA.propBool);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

	it("Should return objA with function mod when merging objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};		

		var f=function pfz(){return "zxy";};

		var res=tjm.merge(objA,{propFunc:f});

		assert.deepEqual(res.propFunc, f);

		res.propInt.should.eql(objA.propInt);
		res.propBool.should.eql(objA.propBool);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
	});
});




describe("TDPJSObjectMerge more complex (multi-object) tests", function()
{
	it("Should return objB when merging objA with objB and an empty object with ObjB having precedence (last arg)", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};	

		var res=tjm.merge(objA, objB, {});

		assert.deepEqual(res.propFunc, objB.propFunc);
	});


	it("Should return objA with int mod when merging objA with objB and an appropriate third object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};

		var res=tjm.merge(objB, objA,{propInt:999});
		res.propInt.should.eql(999);

		res.propBool.should.eql(objA.propBool);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

	it("Should return objA with int and str mods when merging objA with objB and an appropriate third and fourth object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};

		var res=tjm.merge(objB, objA, {propStr:"abcd"}, {propInt:999});

		res.propInt.should.eql(999);
		res.propStr.should.eql("abcd");

		res.propBool.should.eql(objA.propBool);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

	it("Should add a new property to objA with an appropriate third object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};

		var res=tjm.merge(objB, objA, {propNew:"ACE"});

		res.propNew.should.eql("ACE");

		res.propBool.should.eql(objA.propBool);
		res.propInt.should.eql(objA.propInt);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propObj, objA.propObj);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

	it("Should recursively add a new sub-object correctly to objA with an appropriate second object", function()
	{
		objA=
		{
			propBool:true,
			propInt:1,
			propStr:"This is a string v1",
			propObj:
			{
				ca:"cb", c1:123
			},
			propArr:[1,2,3,4,5,6,7,8,9],
			propFunc:function pfa(){return true;}
		};

		objB=
		{
			propBool:false,
			propInt:-1,
			propStr:"This is a string v2",
			propObj:
			{
				ca:"bc", c1:321
			},
			propArr:[9,8,7,6,5,4,3,2,1],
			propFunc:function pfb(){return false;}
		};

		var tstObj={propObj:{newProp:{n:"new", dis:{guy:"alien"}}}};

		var res=tjm.merge(objA, objB, tstObj);

		res.propObj.newProp.n.should.eql("new");
		assert.deepEqual(res.propObj.newProp, tstObj.propObj.newProp);

		res.propBool.should.eql(objA.propBool);
		res.propInt.should.eql(objA.propInt);
		res.propStr.should.eql(objA.propStr);
		assert.deepEqual(res.propArr, objA.propArr);
		assert.deepEqual(res.propFunc, objA.propFunc);
	});

});


